#! /usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const node_process_1 = __importDefault(require("process"));
const list_files_js_1 = require("../modules/list-files.js");
const get_arg_js_1 = require("../helpers/get-arg.js");
const files_js_1 = require("../configs/files.js");
const get_runtime_js_1 = require("../helpers/get-runtime.js");
const format_js_1 = require("../helpers/format.js");
const logs_js_1 = require("../helpers/logs.js");
const hr_js_1 = require("../helpers/hr.js");
const map_tests_js_1 = require("../services/map-tests.js");
const watch_js_1 = require("../services/watch.js");
const poku_js_1 = require("../modules/poku.js");
const processes_js_1 = require("../modules/processes.js");
const dirs = (() => {
    const includeArg = (0, get_arg_js_1.getArg)('include');
    if (includeArg !== undefined)
        return includeArg.split(',');
    const lastParam = (0, get_arg_js_1.getLastParam)();
    if (lastParam !== undefined)
        return lastParam.split(',');
    return ['.'];
})();
const platform = (0, get_arg_js_1.getArg)('platform');
const filter = (0, get_arg_js_1.getArg)('filter');
const exclude = (0, get_arg_js_1.getArg)('exclude');
const killPort = (0, get_arg_js_1.getArg)('kill-port');
const killRange = (0, get_arg_js_1.getArg)('kill-range');
const killPID = (0, get_arg_js_1.getArg)('kill-pid');
const denoAllow = (0, get_arg_js_1.argToArray)('deno-allow');
const denoDeny = (0, get_arg_js_1.argToArray)('deno-deny');
const denoCJS = ((_a = (0, get_arg_js_1.getArg)('deno-cjs')) === null || _a === void 0 ? void 0 : _a.split(',').map((a) => a.trim()).filter((a) => a)) || (0, get_arg_js_1.hasArg)('deno-cjs');
const parallel = (0, get_arg_js_1.hasArg)('parallel');
const quiet = (0, get_arg_js_1.hasArg)('quiet');
const debug = (0, get_arg_js_1.hasArg)('debug');
const failFast = (0, get_arg_js_1.hasArg)('fail-fast');
const watchMode = (0, get_arg_js_1.hasArg)('watch');
const concurrency = parallel
    ? Number((0, get_arg_js_1.getArg)('concurrency')) || undefined
    : undefined;
const tasks = [];
if (killPort) {
    const ports = killPort.split(',').map(Number);
    tasks.push(processes_js_1.kill.port(ports));
}
if (killRange) {
    const ranges = killRange.split(',');
    for (const range of ranges) {
        const ports = range.split('-').map(Number);
        const startsAt = ports[0];
        const endsAt = ports[1];
        tasks.push(processes_js_1.kill.range(startsAt, endsAt));
    }
}
if (killPID) {
    const PIDs = killPID.split(',').map(Number);
    tasks.push(processes_js_1.kill.pid(PIDs));
}
const options = {
    platform: (0, get_runtime_js_1.platformIsValid)(platform) ? platform : undefined,
    filter: filter ? new RegExp((0, list_files_js_1.escapeRegExp)(filter)) : undefined,
    exclude: exclude ? new RegExp((0, list_files_js_1.escapeRegExp)(exclude)) : undefined,
    parallel,
    quiet,
    debug,
    failFast,
    concurrency,
    noExit: watchMode,
    deno: {
        allow: denoAllow,
        deny: denoDeny,
        cjs: denoCJS,
    },
};
if (debug) {
    (0, hr_js_1.hr)();
    (0, logs_js_1.write)(`${(0, format_js_1.format)(' Debug Enabled ').bg('brightBlue')}\n`);
    (0, logs_js_1.write)(`${(0, format_js_1.format)('…').info().italic()} ${(0, format_js_1.format)('Paths').bold()}`);
    console.table(dirs);
    (0, logs_js_1.write)('\n');
    (0, logs_js_1.write)(`${(0, format_js_1.format)('…').info().italic()} ${(0, format_js_1.format)('Options').bold()}`);
    console.dir(options, { depth: null, colors: true });
}
Promise.all(tasks).then(() => {
    (0, poku_js_1.poku)(dirs, options).then(() => {
        if (watchMode) {
            const executing = new Set();
            const interval = Number((0, get_arg_js_1.getArg)('watch-interval')) || 1500;
            const resultsClear = () => {
                files_js_1.fileResults.success.clear();
                files_js_1.fileResults.fail.clear();
            };
            node_process_1.default.removeListener('SIGINT', poku_js_1.onSigint);
            resultsClear();
            (0, map_tests_js_1.mapTests)('.', dirs, options.filter, options.exclude).then((mappedTests) => {
                Array.from(mappedTests.keys()).forEach((mappedTest) => {
                    (0, watch_js_1.watch)(mappedTest, (file, event) => {
                        if (event === 'change') {
                            const filePath = (0, map_tests_js_1.normalizePath)(file);
                            if (executing.has(filePath))
                                return;
                            executing.add(filePath);
                            resultsClear();
                            const tests = mappedTests.get(filePath);
                            if (!tests)
                                return;
                            (0, poku_js_1.poku)(Array.from(tests), options).then(() => {
                                setTimeout(() => {
                                    executing.delete(filePath);
                                }, interval);
                            });
                        }
                    });
                });
            });
            dirs.forEach((dir) => {
                (0, watch_js_1.watch)(dir, (file, event) => {
                    if (event === 'change') {
                        if (executing.has(file))
                            return;
                        executing.add(file);
                        resultsClear();
                        (0, poku_js_1.poku)(file, options).then(() => {
                            setTimeout(() => {
                                executing.delete(file);
                            }, interval);
                        });
                    }
                });
            });
            (0, hr_js_1.hr)();
            (0, logs_js_1.write)(`${(0, format_js_1.format)('Watching:').bold()} ${(0, format_js_1.format)(dirs.join(', ')).underline()}`);
        }
    });
});
