"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.argToArray = exports.getLastParam = exports.hasArg = exports.getArg = void 0;
/* c8 ignore start */
const node_process_1 = require("process");
const [, , ...processArgs] = node_process_1.argv;
/**
 * Gets the value of an argument.
 *
 * ---
 *
 * CLI arguments examples:
 *
 * ```sh
 * command --arg=some # 'some'
 * command --arg=""   # ''
 * command --arg      # undefined
 * ```
 */
const getArg = (arg, prefix = '--') => {
    const argPattern = `${prefix}${arg}=`;
    const argValue = processArgs.find((a) => a.startsWith(argPattern));
    if (!argValue)
        return undefined;
    return argValue.slice(argPattern.length).replace(/''|""/, '');
};
exports.getArg = getArg;
/**
 * Checks if an argument exists.
 *
 * ---
 *
 * CLI arguments examples:
 *
 * ```sh
 * command --arg  # true
 * command        # false
 * ```
 */
const hasArg = (arg, prefix = '--') => {
    const argPattern = `${prefix}${arg}`;
    return processArgs.some((a) => a.startsWith(argPattern));
};
exports.hasArg = hasArg;
/**
 * Gets the last param/value.
 *
 * CLI arguments examples:
 *
 * ```sh
 * command --arg --arg2=some value  # 'value'
 * command value                    # 'value'
 * command                          # undefined
 * command --arg                    # undefined
 * ```
 */
const getLastParam = (prefix = '--') => {
    const lastArg = processArgs[processArgs.length - 1];
    if (!lastArg || lastArg.startsWith(prefix))
        return undefined;
    return lastArg;
};
exports.getLastParam = getLastParam;
const argToArray = (arg, prefix = '--') => {
    const hasArgument = (0, exports.hasArg)(arg);
    if (!hasArgument)
        return undefined;
    const argValue = (0, exports.getArg)(arg, prefix);
    if (hasArgument && !argValue)
        return [];
    if (!argValue)
        return undefined;
    return argValue
        .split(',')
        .map((a) => a.trim())
        .filter((a) => a);
};
exports.argToArray = argToArray;
/* c8 ignore stop */
