"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseAssertion = exports.parseResultType = void 0;
const node_process_1 = require("process");
const node_path_1 = __importDefault(require("path"));
const node_assert_1 = __importDefault(require("assert"));
const format_js_1 = require("./format.js");
const hr_js_1 = require("./hr.js");
const find_file_js_1 = require("./find-file.js");
const each_js_1 = require("../configs/each.js");
const indentation_js_1 = require("../configs/indentation.js");
const object_js_1 = require("../polyfills/object.js");
const get_runtime_js_1 = require("./get-runtime.js");
const logs_js_1 = require("./logs.js");
const cwd = (0, node_process_1.cwd)();
const parseResultType = (type) => {
    const recurse = (value) => {
        if (typeof value === 'undefined' ||
            typeof value === 'function' ||
            typeof value === 'bigint' ||
            typeof value === 'symbol' ||
            value instanceof RegExp)
            return String(value);
        if (Array.isArray(value))
            return value.map(recurse);
        if (value instanceof Set)
            return Array.from(value).map(recurse);
        /* c8 ignore start */
        if (value instanceof Map)
            return recurse(!get_runtime_js_1.nodeVersion || get_runtime_js_1.nodeVersion >= 12
                ? Object.fromEntries(value)
                : (0, object_js_1.fromEntries)(value));
        /* c8 ignore stop */
        /* c8 ignore start */
        if (value !== null && typeof value === 'object') {
            if (!get_runtime_js_1.nodeVersion || get_runtime_js_1.nodeVersion >= 12)
                return Object.fromEntries(Object.entries(value).map(([key, val]) => [key, recurse(val)]));
            return (0, object_js_1.fromEntries)((0, object_js_1.entries)(value).map(([key, val]) => [key, recurse(val)]));
        }
        /* c8 ignore stop */
        return value;
    };
    const result = recurse(type);
    return typeof result === 'string' ? result : JSON.stringify(result, null, 2);
};
exports.parseResultType = parseResultType;
const parseAssertion = (cb, options) => __awaiter(void 0, void 0, void 0, function* () {
    const isPoku = typeof (node_process_1.env === null || node_process_1.env === void 0 ? void 0 : node_process_1.env.FILE) === 'string' && (node_process_1.env === null || node_process_1.env === void 0 ? void 0 : node_process_1.env.FILE.length) > 0;
    const FILE = node_process_1.env.FILE;
    let preIdentation = '';
    if (indentation_js_1.indentation.hasDescribe || indentation_js_1.indentation.hasTest)
        preIdentation += '  ';
    if (indentation_js_1.indentation.hasIt)
        preIdentation += '  ';
    try {
        if (typeof each_js_1.each.before.cb === 'function' && each_js_1.each.before.assert) {
            const beforeResult = each_js_1.each.before.cb();
            /* c8 ignore next */
            if (beforeResult instanceof Promise)
                yield beforeResult;
            /* c8 ignore next */
        }
        const cbResult = cb();
        if (cbResult instanceof Promise)
            yield cbResult;
        if (typeof each_js_1.each.after.cb === 'function' && each_js_1.each.after.assert) {
            const afterResult = each_js_1.each.after.cb();
            /* c8 ignore next */
            if (afterResult instanceof Promise)
                yield afterResult;
            /* c8 ignore next */
        }
        if (typeof options.message === 'string') {
            const message = isPoku &&
                !indentation_js_1.indentation.hasDescribe &&
                !indentation_js_1.indentation.hasIt &&
                /* c8 ignore next */ // c8 bug?
                !indentation_js_1.indentation.hasTest
                ? /* c8 ignore next */
                    `${preIdentation}${(0, format_js_1.format)(`${(0, format_js_1.format)(`✔ ${options.message}`).bold()} ${(0, format_js_1.format)(`› ${FILE}`).success().dim()}`).success()}`
                : `${preIdentation}${(0, format_js_1.format)(`✔ ${options.message}`).success().bold()}`;
            (0, logs_js_1.write)(message);
        }
        /* c8 ignore start */
    }
    catch (error) {
        if (error instanceof node_assert_1.default.AssertionError) {
            const { code, actual, expected, operator } = error;
            const absoultePath = (0, find_file_js_1.findFile)(error).replace(/file:(\/\/)?/, '');
            const file = node_path_1.default.relative(node_path_1.default.resolve(cwd), absoultePath);
            let message = '';
            if (typeof options.message === 'string')
                message = options.message;
            else if (options.message instanceof Error)
                message = options.message.message;
            else if (typeof options.defaultMessage === 'string')
                message = options.defaultMessage;
            const finalMessage = (message === null || message === void 0 ? void 0 : message.trim().length) > 0
                ? (0, format_js_1.format)(`✘ ${message}`).fail().bold()
                : (0, format_js_1.format)('✘ No Message').fail().bold();
            (0, logs_js_1.write)(isPoku
                ? `${preIdentation}${finalMessage} ${(0, format_js_1.format)(`› ${FILE}`).fail().dim()}`
                : `${preIdentation}${finalMessage}`);
            file && (0, logs_js_1.write)(`${(0, format_js_1.format)(`${preIdentation}      File`).dim()} ${file}`);
            (0, logs_js_1.write)(`${(0, format_js_1.format)(`${preIdentation}      Code`).dim()} ${code}`);
            (0, logs_js_1.write)(`${(0, format_js_1.format)(`${preIdentation}  Operator`).dim()} ${operator}\n`);
            if (!(options === null || options === void 0 ? void 0 : options.hideDiff)) {
                const splitActual = (0, exports.parseResultType)(actual).split('\n');
                const splitExpected = (0, exports.parseResultType)(expected).split('\n');
                (0, logs_js_1.write)((0, format_js_1.format)(`${preIdentation}  ${(options === null || options === void 0 ? void 0 : options.actual) || 'Actual'}:`).dim());
                splitActual.forEach((line) => (0, logs_js_1.write)(`${preIdentation}  ${(0, format_js_1.format)(line).fail().bold()}`));
                (0, logs_js_1.write)(`\n${preIdentation}  ${(0, format_js_1.format)(`${(options === null || options === void 0 ? void 0 : options.expected) || 'Expected'}:`).dim()}`);
                splitExpected.forEach((line) => (0, logs_js_1.write)(`${preIdentation}  ${(0, format_js_1.format)(line).success().bold()}`));
            }
            if (options.throw) {
                console.error(error);
                (0, hr_js_1.hr)();
            }
            (0, node_process_1.exit)(1);
        }
        // Non-assertion errors
        throw error;
    }
    /* c8 ignore stop */
});
exports.parseAssertion = parseAssertion;
