"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scriptRunner = exports.runner = exports.isWindows = void 0;
/* c8 ignore start */
const node_process_1 = require("process");
const node_path_1 = require("path");
const get_runtime_js_1 = require("./get-runtime.js");
exports.isWindows = node_process_1.platform === 'win32';
const runner = (filename, configs) => {
    var _a, _b;
    const runtime = (0, get_runtime_js_1.getRuntime)(configs);
    // Bun
    if (runtime === 'bun')
        return ['bun'];
    // Deno
    if (runtime === 'deno') {
        const denoAllow = ((_a = configs === null || configs === void 0 ? void 0 : configs.deno) === null || _a === void 0 ? void 0 : _a.allow)
            ? configs.deno.allow
                .map((allow) => (allow ? `--allow-${allow}` : ''))
                .filter((allow) => allow)
            : [
                // Defaults
                '--allow-read', // Poku searches for all test files
                '--allow-env', // Poku share the process.env with the `child_process`
                '--allow-run', // Poku CLI
                '--allow-net', // Create Service
                '--allow-hrtime', // Precise performance execution time
            ];
        const denoDeny = ((_b = configs === null || configs === void 0 ? void 0 : configs.deno) === null || _b === void 0 ? void 0 : _b.deny)
            ? configs.deno.deny
                .map((deny) => (deny ? `--deny-${deny}` : ''))
                .filter((deny) => deny)
            : [];
        return ['deno', 'run', ...denoAllow, ...denoDeny];
    }
    // Node.js
    return (0, node_path_1.extname)(filename) === '.ts'
        ? [exports.isWindows ? 'npx.cmd' : 'npx', 'tsx']
        : ['node'];
};
exports.runner = runner;
const scriptRunner = (runner) => {
    // Bun
    if (runner === 'bun')
        return ['bun', 'run'];
    // Deno
    if (runner === 'deno')
        return ['deno', 'task'];
    // Yarn
    if (runner === 'yarn')
        return ['yarn'];
    // PNPM
    if (runner === 'pnpm')
        return ['pnpm', 'run'];
    // Node.js
    return [exports.isWindows ? 'npm.cmd' : 'npm', 'run'];
};
exports.scriptRunner = scriptRunner;
/* c8 ignore stop */
