import type { Control, EachOptions } from '../@types/each.js';
/**
 * - ✅ Handling **global** and **external** services (_preparing a database, for example_)
 * - ✅ It's made for **exclusive use** in combination with **Poku**'s **`assert`** methods
 * - ⚠️ Although `beforeEach` accepts local variables changes by using the `imediate` option, it's strongly discouraged ([_use a mock instead_](https://poku.io/docs/category/mock))
 *
 * ---
 *
 * ### Available methos:
 *
 * ```ts
 * import { assert, beforeEach } from 'poku';
 *
 * const before = beforeEach(() => {
 *   // preparing an external or global state
 * };
 *
 * before.pause();
 * before.continue();
 * before.reset();
 * ```
 */
export declare const beforeEach: (callback: () => unknown, options?: EachOptions) => Control;
/**
 * - ✅ Handling **global** and **external** services (_preparing a database, for example_)
 * - ✅ It's made for **exclusive use** in combination with **Poku**'s **`assert`** methods
 * - ⚠️ Although `afterEach` accepts local variables changes, it's strongly discouraged ([_use a mock instead_](https://poku.io/docs/category/mock))
 *
 * ---
 *
 * ### Available methos:
 *
 * ```ts
 * import { assert, afterEach } from 'poku';
 *
 * const after = afterEach(() => {
 *   // cleanup an external or global state
 * };
 *
 * after.pause();
 * after.continue();
 * after.reset();
 * ```
 */
export declare const afterEach: (callback: () => unknown, options?: Omit<EachOptions, "immediate">) => Control;
