"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.afterEach = exports.beforeEach = void 0;
const each_js_1 = require("../configs/each.js");
/**
 * - ✅ Handling **global** and **external** services (_preparing a database, for example_)
 * - ✅ It's made for **exclusive use** in combination with **Poku**'s **`assert`** methods
 * - ⚠️ Although `beforeEach` accepts local variables changes by using the `imediate` option, it's strongly discouraged ([_use a mock instead_](https://poku.io/docs/category/mock))
 *
 * ---
 *
 * ### Available methos:
 *
 * ```ts
 * import { assert, beforeEach } from 'poku';
 *
 * const before = beforeEach(() => {
 *   // preparing an external or global state
 * };
 *
 * before.pause();
 * before.continue();
 * before.reset();
 * ```
 */
const beforeEach = (callback, options) => {
    each_js_1.each.before.test = typeof (options === null || options === void 0 ? void 0 : options.test) === 'boolean' ? options.test : true;
    each_js_1.each.before.assert =
        typeof (options === null || options === void 0 ? void 0 : options.assert) === 'boolean' ? options.assert : false;
    (options === null || options === void 0 ? void 0 : options.immediate) && callback();
    each_js_1.each.before.cb = () => {
        if (each_js_1.each.before.status)
            callback();
    };
    const pause = () => {
        each_js_1.each.before.status = false;
    };
    const continueFunc = () => {
        each_js_1.each.before.status = true;
    };
    const reset = () => {
        each_js_1.each.before.cb = undefined;
    };
    return { pause, continue: continueFunc, reset };
};
exports.beforeEach = beforeEach;
/* c8 ignore start */ // c8 bug
/**
 * - ✅ Handling **global** and **external** services (_preparing a database, for example_)
 * - ✅ It's made for **exclusive use** in combination with **Poku**'s **`assert`** methods
 * - ⚠️ Although `afterEach` accepts local variables changes, it's strongly discouraged ([_use a mock instead_](https://poku.io/docs/category/mock))
 *
 * ---
 *
 * ### Available methos:
 *
 * ```ts
 * import { assert, afterEach } from 'poku';
 *
 * const after = afterEach(() => {
 *   // cleanup an external or global state
 * };
 *
 * after.pause();
 * after.continue();
 * after.reset();
 * ```
 */
const afterEach = (callback, options) => {
    /* c8 ignore stop */
    each_js_1.each.after.test = typeof (options === null || options === void 0 ? void 0 : options.test) === 'boolean' ? options.test : true;
    each_js_1.each.after.assert =
        typeof (options === null || options === void 0 ? void 0 : options.assert) === 'boolean' ? options.assert : false;
    each_js_1.each.after.cb = () => {
        if (each_js_1.each.after.status)
            callback();
    };
    const pause = () => {
        each_js_1.each.after.status = false;
    };
    const continueFunc = () => {
        each_js_1.each.after.status = true;
    };
    const reset = () => {
        each_js_1.each.after.cb = undefined;
    };
    return { pause, continue: continueFunc, reset };
};
exports.afterEach = afterEach;
