"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.listFiles = exports.getAllFiles = exports.escapeRegExp = exports.isFile = exports.sanitizePath = void 0;
const node_process_1 = require("process");
const node_path_1 = require("path");
const fs_js_1 = require("../polyfills/fs.js");
const sanitizePath = (input, ensureTarget) => {
    const sanitizedPath = input
        .replace(/[/\\]+/g, node_path_1.sep) // adapting slashes according to OS
        .replace(/(\.\.(\/|\\|$))+/g, '') // ensure the current path level
        .replace(/[<>|^?*]+/g, ''); // removing unusual path characters
    // Preventing absolute path access
    return ensureTarget
        ? sanitizedPath.replace(/^[/\\]/, `.${node_path_1.sep}`)
        : sanitizedPath;
};
exports.sanitizePath = sanitizePath;
/* c8 ignore start */
const isFile = (fullPath) => __awaiter(void 0, void 0, void 0, function* () { return (yield (0, fs_js_1.stat)(fullPath)).isFile(); });
exports.isFile = isFile;
/* c8 ignore stop */
/* c8 ignore start */
const escapeRegExp = (string) => string.replace(/[.*{}[\]\\]/g, '\\$&');
exports.escapeRegExp = escapeRegExp;
/* c8 ignore stop */
/* c8 ignore start */
const envFilter = ((_a = node_process_1.env.FILTER) === null || _a === void 0 ? void 0 : _a.trim())
    ? new RegExp((0, exports.escapeRegExp)(node_process_1.env.FILTER), 'i')
    : undefined;
/* c8 ignore stop */
const getAllFiles = (dirPath_1, ...args_1) => __awaiter(void 0, [dirPath_1, ...args_1], void 0, function* (dirPath, files = new Set(), configs) {
    const currentFiles = yield (0, fs_js_1.readdir)((0, exports.sanitizePath)(dirPath));
    const defaultRegExp = /\.(test|spec)\./i;
    /* c8 ignore start */
    const filter = (envFilter
        ? envFilter
        : (configs === null || configs === void 0 ? void 0 : configs.filter) instanceof RegExp
            ? configs.filter
            : defaultRegExp) || defaultRegExp;
    const exclude = (configs === null || configs === void 0 ? void 0 : configs.exclude)
        ? Array.isArray(configs.exclude)
            ? configs.exclude
            : [configs.exclude]
        : undefined;
    /* c8 ignore stop */
    yield Promise.all(currentFiles.map((file) => __awaiter(void 0, void 0, void 0, function* () {
        const fullPath = (0, node_path_1.join)(dirPath, file);
        const stat = yield (0, fs_js_1.stat)(fullPath);
        /* c8 ignore start */
        if (fullPath.indexOf('node_modules') !== -1 ||
            fullPath.indexOf('.git') === 0)
            return;
        /* c8 ignore stop */
        if (exclude) {
            for (let i = 0; i < exclude.length; i++) {
                /* c8 ignore next */
                if (exclude[i].test(fullPath))
                    return;
            }
        }
        if (filter.test(fullPath))
            return files.add(fullPath);
        if (stat.isDirectory())
            yield (0, exports.getAllFiles)(fullPath, files, configs);
    })));
    return files;
});
exports.getAllFiles = getAllFiles;
/* c8 ignore start */ // c8 bug
const listFiles = (targetDir, configs) => __awaiter(void 0, void 0, void 0, function* () { return Array.from(yield (0, exports.getAllFiles)((0, exports.sanitizePath)(targetDir), new Set(), configs)); });
exports.listFiles = listFiles;
/* c8 ignore stop */
