"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onSigint = void 0;
exports.poku = poku;
/* c8 ignore start */ // c8 bug (incompatibility) =>
/**
 * Both CLI, API, noExit, sequential and parallel runs are strictly tested, but these tests use deep child process for it
 */
const node_process_1 = __importDefault(require("process"));
const run_tests_js_1 = require("../services/run-tests.js");
const exit_js_1 = require("./exit.js");
const format_js_1 = require("../helpers/format.js");
const logs_js_1 = require("../helpers/logs.js");
const hr_js_1 = require("../helpers/hr.js");
const files_js_1 = require("../configs/files.js");
const indentation_js_1 = require("../configs/indentation.js");
const onSigint = () => {
    node_process_1.default.stdout.write('\u001B[?25h');
};
exports.onSigint = onSigint;
node_process_1.default.once('SIGINT', exports.onSigint);
function poku(targetPaths, configs) {
    return __awaiter(this, void 0, void 0, function* () {
        let code = 0;
        files_js_1.finalResults.started = new Date();
        const start = node_process_1.default.hrtime();
        const prepareDirs = Array.prototype.concat(targetPaths);
        const dirs = prepareDirs.length > 0 ? prepareDirs : ['.'];
        const showLogs = !(0, logs_js_1.isQuiet)(configs);
        // Sequential
        if (!(configs === null || configs === void 0 ? void 0 : configs.parallel)) {
            for (const dir of dirs) {
                const result = yield (0, run_tests_js_1.runTests)(dir, configs);
                if (!result) {
                    code = 1;
                    if (configs === null || configs === void 0 ? void 0 : configs.failFast)
                        break;
                }
            }
            if (configs === null || configs === void 0 ? void 0 : configs.noExit)
                return code;
            const end = node_process_1.default.hrtime(start);
            const total = (end[0] * 1e3 + end[1] / 1e6).toFixed(6);
            files_js_1.finalResults.time = total;
            (0, exit_js_1.exit)(code, configs === null || configs === void 0 ? void 0 : configs.quiet);
            return;
        }
        // Parallel
        if (showLogs) {
            (0, hr_js_1.hr)();
            (0, logs_js_1.write)(`${(0, format_js_1.format)('Running the Test Suite in Parallel').bold()}\n`);
        }
        try {
            const promises = dirs.map((dir) => __awaiter(this, void 0, void 0, function* () {
                const result = yield (0, run_tests_js_1.runTestsParallel)(dir, configs);
                if (!result && (configs === null || configs === void 0 ? void 0 : configs.failFast))
                    throw '';
                return result;
            }));
            const concurrency = yield Promise.all(promises);
            if (concurrency.some((result) => !result))
                code = 1;
        }
        catch (_a) {
        }
        finally {
            const end = node_process_1.default.hrtime(start);
            const total = (end[0] * 1e3 + end[1] / 1e6).toFixed(6);
            files_js_1.finalResults.time = total;
        }
        showLogs && (0, hr_js_1.hr)();
        if (showLogs && files_js_1.fileResults.success.size > 0) {
            (0, logs_js_1.write)(Array.from(files_js_1.fileResults.success)
                .map(([file, time]) => `${indentation_js_1.indentation.test}${(0, format_js_1.format)('✔').success()} ${(0, format_js_1.format)(`${file} ${(0, format_js_1.format)(`› ${time}ms`).success()}`).dim()}`)
                .join('\n'));
        }
        if (showLogs && files_js_1.fileResults.fail.size > 0) {
            (0, logs_js_1.write)(Array.from(files_js_1.fileResults.fail)
                .map(([file, time]) => `${indentation_js_1.indentation.test}${(0, format_js_1.format)('✘').fail()} ${(0, format_js_1.format)(`${file} ${(0, format_js_1.format)(`› ${time}ms`).fail()}`).dim()}`)
                .join('\n'));
        }
        if (configs === null || configs === void 0 ? void 0 : configs.noExit)
            return code;
        (0, exit_js_1.exit)(code, configs === null || configs === void 0 ? void 0 : configs.quiet);
    });
}
/* c8 ignore stop */
