"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.waitForPort = exports.sleep = void 0;
const node_net_1 = require("net");
const checkPort = (port, host) => new Promise((resolve) => {
    const client = (0, node_net_1.createConnection)(port, host);
    client.on('connect', () => {
        client.destroy();
        resolve(true);
    });
    /* c8 ignore start */
    client.on('error', () => {
        resolve(false);
    });
    /* c8 ignore stop */
});
/**
 * Wait until the defined milliseconds.
 */
const sleep = (milliseconds) => {
    /* c8 ignore start */
    if (!Number.isInteger(milliseconds)) {
        throw new Error(`Milliseconds must be an integer.`);
    }
    /* c8 ignore stop */
    return new Promise((resolve) => setTimeout(resolve, milliseconds));
};
exports.sleep = sleep;
/* c8 ignore start */ // c8 bug
/**
 * Wait until the defined port is active.
 */
const waitForPort = (port, options) => __awaiter(void 0, void 0, void 0, function* () {
    /* c8 ignore stop */
    const delay = (options === null || options === void 0 ? void 0 : options.delay) || 0;
    const interval = (options === null || options === void 0 ? void 0 : options.interval) || 100;
    const timeout = (options === null || options === void 0 ? void 0 : options.timeout) || 60000;
    const host = (options === null || options === void 0 ? void 0 : options.host) || 'localhost';
    /* c8 ignore start */
    if (!Number.isInteger(port)) {
        throw new Error('Port must be an integer.');
    }
    if (!Number.isInteger(interval)) {
        throw new Error('Interval must be an integer.');
    }
    if (!Number.isInteger(timeout)) {
        throw new Error('Timeout must be an integer.');
    }
    if (!Number.isInteger(delay)) {
        throw new Error('Delay must be an integer.');
    }
    /* c8 ignore stop */
    yield (0, exports.sleep)(delay);
    const startTime = Date.now();
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const hasPort = yield checkPort(port, host);
        if (hasPort)
            break;
        /* c8 ignore start */
        if (Date.now() - startTime >= timeout) {
            throw new Error(`Timeout waiting for port ${port} to become active`);
        }
        /* c8 ignore stop */
        yield (0, exports.sleep)(interval);
    }
    yield (0, exports.sleep)(delay);
});
exports.waitForPort = waitForPort;
