"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DockerCompose = exports.DockerContainer = void 0;
/* c8 ignore next */ // c8 bug
const node_child_process_1 = require("child_process");
const logs_js_1 = require("../helpers/logs.js");
const runner_js_1 = require("../helpers/runner.js");
const list_files_js_1 = require("../modules/list-files.js");
const runDockerCommand = (command, args, options, verbose) => {
    return new Promise((resolve) => {
        const dockerProcess = (0, node_child_process_1.spawn)(command, args, Object.assign(Object.assign({}, options), { shell: runner_js_1.isWindows }));
        /* c8 ignore start */
        if (verbose) {
            dockerProcess.stdout.on('data', logs_js_1.write);
            dockerProcess.stderr.on('data', logs_js_1.write);
        }
        /* c8 ignore stop */
        dockerProcess.on('close', (code) => {
            resolve(code === 0);
        });
        /* c8 ignore start */
        dockerProcess.on('error', () => {
            resolve(false);
        });
        /* c8 ignore stop */
    });
};
class DockerContainer {
    constructor(configs) {
        const { context, file, tagName, containerName, ports, environments, cache, detach, envFile, cwd, verbose, } = configs;
        this.tagName = tagName;
        this.containerName = containerName;
        this.file = file || './Dockerfile';
        this.context = context || '.';
        this.ports = ports || [];
        this.cache = cache;
        this.environments = environments || [];
        this.envFile = envFile;
        this.detach = detach;
        /* c8 ignore next */
        this.cwd = cwd ? (0, list_files_js_1.sanitizePath)(cwd) : undefined;
        this.verbose = verbose;
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            const args = ['build'];
            if (this.cache === false)
                args.push('--no-cache');
            yield runDockerCommand('docker', [...args, '-t', this.tagName, '-f', this.file, this.context], { cwd: this.cwd }, this.verbose);
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const args = ['run'];
            /* c8 ignore next */
            args.push(this.detach !== false ? '-d' : '--init');
            args.push(...['--name', this.containerName]);
            this.ports.forEach((port) => args.push(...['-p', port]));
            this.environments.forEach((environment) => args.push(...['-e', environment]));
            if (this.envFile)
                args.push(...['--env-file', this.envFile]);
            return yield runDockerCommand('docker', [...args, this.tagName], { cwd: this.cwd }, this.verbose);
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield runDockerCommand('docker', ['stop', this.containerName], { cwd: this.cwd }, this.verbose);
        });
    }
    remove() {
        return __awaiter(this, void 0, void 0, function* () {
            yield runDockerCommand('docker', ['rm', '-f', this.containerName], { cwd: this.cwd }, this.verbose);
            yield runDockerCommand('docker', ['image', 'rm', '-f', this.tagName], { cwd: this.cwd }, this.verbose);
        });
    }
}
exports.DockerContainer = DockerContainer;
class DockerCompose {
    constructor(configs) {
        const { file, projectName, build, serviceName, envFile, detach, cwd, verbose, } = configs;
        this.file = file || './docker-compose.yml';
        this.build = build;
        this.serviceName = serviceName;
        this.envFile = envFile;
        this.projectName = projectName;
        this.detach = detach;
        /* c8 ignore next */
        this.cwd = cwd ? (0, list_files_js_1.sanitizePath)(cwd) : undefined;
        this.verbose = verbose;
    }
    up() {
        return __awaiter(this, void 0, void 0, function* () {
            const args = ['-f', this.file];
            if (this.envFile)
                args.push(...['--env-file', this.envFile]);
            if (this.projectName)
                args.push(...['-p', this.projectName]);
            args.push('up');
            /* c8 ignore next */
            args.push(this.detach !== false ? '-d' : '--abort-on-container-exit');
            if (this.build)
                args.push('--build');
            if (this.serviceName)
                args.push(this.serviceName);
            return yield runDockerCommand('docker-compose', args, { cwd: this.cwd }, this.verbose);
        });
    }
    down() {
        return __awaiter(this, void 0, void 0, function* () {
            const args = ['-f', this.file];
            if (this.envFile)
                args.push(...['--env-file', this.envFile]);
            if (this.projectName)
                args.push(...['-p', this.projectName]);
            return yield runDockerCommand('docker-compose', [...args, 'down'], { cwd: this.cwd }, this.verbose);
        });
    }
}
exports.DockerCompose = DockerCompose;
