"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapTests = exports.findMatchingFiles = exports.getDeepImports = exports.normalizePath = void 0;
/* c8 ignore next */ // c8 bug
const node_path_1 = require("path");
const fs_js_1 = require("../polyfills/fs.js");
const list_files_js_1 = require("../modules/list-files.js");
const importMap = new Map();
const processedFiles = new Set();
const extFilter = /\.(js|cjs|mjs|ts|cts|mts|jsx|tsx)$/;
const normalizePath = (filePath) => filePath
    .replace(/(\.\/)/g, '')
    .replace(/^\.+/, '')
    .replace(/[/\\]+/g, node_path_1.sep)
    .replace(/\\/g, '/');
exports.normalizePath = normalizePath;
const getDeepImports = (content) => {
    const paths = new Set();
    const lines = content.split('\n');
    for (const line of lines) {
        if (line.includes('import') || line.includes('require')) {
            const path = line.match(/['"](\.{1,2}\/[^'"]+)['"]/);
            if (path)
                paths.add((0, exports.normalizePath)(path[1].replace(extFilter, '')));
        }
    }
    return paths;
};
exports.getDeepImports = getDeepImports;
const findMatchingFiles = (srcFilesWithoutExt, srcFilesWithExt) => {
    const matchingFiles = new Set();
    srcFilesWithoutExt.forEach((srcFile) => {
        const normalizedSrcFile = (0, exports.normalizePath)(srcFile);
        srcFilesWithExt.forEach((fileWithExt) => {
            const normalizedFileWithExt = (0, exports.normalizePath)(fileWithExt);
            if (normalizedFileWithExt.includes(normalizedSrcFile))
                matchingFiles.add(fileWithExt);
        });
    });
    return matchingFiles;
};
exports.findMatchingFiles = findMatchingFiles;
/* c8 ignore start */
const collectTestFiles = (testPaths, testFilter, exclude) => __awaiter(void 0, void 0, void 0, function* () {
    const statsPromises = testPaths.map((testPath) => (0, fs_js_1.stat)(testPath));
    const stats = yield Promise.all(statsPromises);
    const listFilesPromises = stats.map((stat, index) => {
        const testPath = testPaths[index];
        if (stat.isDirectory())
            return (0, list_files_js_1.listFiles)(testPath, {
                filter: testFilter,
                exclude,
            });
        if (stat.isFile() && extFilter.test(testPath))
            return [testPath];
        else
            return [];
    });
    const nestedTestFiles = yield Promise.all(listFilesPromises);
    return new Set(nestedTestFiles.flat());
});
/* c8 ignore stop */
/* c8 ignore start */
const processDeepImports = (srcFile, testFile, intersectedSrcFiles) => __awaiter(void 0, void 0, void 0, function* () {
    if (processedFiles.has(srcFile))
        return;
    processedFiles.add(srcFile);
    const srcContent = yield (0, fs_js_1.readFile)(srcFile, 'utf-8');
    const deepImports = (0, exports.getDeepImports)(srcContent);
    const matchingFiles = (0, exports.findMatchingFiles)(deepImports, intersectedSrcFiles);
    for (const deepImport of matchingFiles) {
        if (!importMap.has(deepImport))
            importMap.set(deepImport, new Set());
        importMap.get(deepImport).add((0, exports.normalizePath)(testFile));
        yield processDeepImports(deepImport, testFile, intersectedSrcFiles);
    }
});
/* c8 ignore stop */
const createImportMap = (allTestFiles, allSrcFiles) => __awaiter(void 0, void 0, void 0, function* () {
    const intersectedSrcFiles = new Set(Array.from(allSrcFiles).filter((srcFile) => !allTestFiles.has(srcFile)));
    yield Promise.all(Array.from(allTestFiles).map((testFile) => __awaiter(void 0, void 0, void 0, function* () {
        const content = yield (0, fs_js_1.readFile)(testFile, 'utf-8');
        for (const srcFile of intersectedSrcFiles) {
            const relativePath = (0, exports.normalizePath)((0, node_path_1.relative)((0, node_path_1.dirname)(testFile), srcFile));
            const normalizedSrcFile = (0, exports.normalizePath)(srcFile);
            /* c8 ignore start */
            if (content.includes(relativePath.replace(extFilter, '')) ||
                content.includes(normalizedSrcFile)) {
                if (!importMap.has(normalizedSrcFile))
                    importMap.set(normalizedSrcFile, new Set());
                importMap.get(normalizedSrcFile).add((0, exports.normalizePath)(testFile));
                yield processDeepImports(srcFile, testFile, intersectedSrcFiles);
            }
            /* c8 ignore stop */
        }
    })));
});
/* c8 ignore next */ // c8 bug
const mapTests = (srcDir, testPaths, testFilter, exclude) => __awaiter(void 0, void 0, void 0, function* () {
    const [allTestFiles, allSrcFiles] = yield Promise.all([
        collectTestFiles(testPaths, testFilter, exclude),
        (0, list_files_js_1.listFiles)(srcDir, {
            filter: extFilter,
            exclude,
        }),
    ]);
    yield createImportMap(allTestFiles, new Set(allSrcFiles));
    return importMap;
});
exports.mapTests = mapTests;
