"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runTestFile = void 0;
const node_process_1 = require("process");
const node_path_1 = require("path");
const node_child_process_1 = require("child_process");
const indentation_js_1 = require("../configs/indentation.js");
const files_js_1 = require("../configs/files.js");
const runner_js_1 = require("../helpers/runner.js");
const format_js_1 = require("../helpers/format.js");
const logs_js_1 = require("../helpers/logs.js");
const each_js_1 = require("./each.js");
const cwd = (0, node_process_1.cwd)();
/* c8 ignore next */ // c8 bug
const runTestFile = (filePath, configs) => new Promise((resolve) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c;
    /* c8 ignore start */
    const runtimeOptions = (0, runner_js_1.runner)(filePath, configs);
    const runtime = runtimeOptions.shift();
    const runtimeArguments = [
        ...runtimeOptions,
        ((_a = configs === null || configs === void 0 ? void 0 : configs.deno) === null || _a === void 0 ? void 0 : _a.cjs) === true ||
            (Array.isArray((_b = configs === null || configs === void 0 ? void 0 : configs.deno) === null || _b === void 0 ? void 0 : _b.cjs) &&
                configs.deno.cjs.some((ext) => filePath.includes(ext)))
            ? 'https://cdn.jsdelivr.net/npm/poku/lib/polyfills/deno.mjs'
            : filePath,
    ];
    /* c8 ignore stop */
    const fileRelative = (0, node_path_1.relative)(cwd, filePath);
    const showLogs = !(0, logs_js_1.isQuiet)(configs);
    let output = '';
    const stdOut = (data) => {
        output += String(data);
    };
    if (!(configs === null || configs === void 0 ? void 0 : configs.parallel)) {
        showLogs &&
            (0, logs_js_1.write)(`${indentation_js_1.indentation.test}${(0, format_js_1.format)('●').info().dim()} ${(0, format_js_1.format)(fileRelative).dim()}`);
    }
    const start = (0, node_process_1.hrtime)();
    let end;
    /* c8 ignore next */
    if (!(yield (0, each_js_1.beforeEach)(fileRelative, configs)))
        return false;
    // Export spawn helper is not an option
    const child = (0, node_child_process_1.spawn)(runtime, runtimeArguments, {
        stdio: ['inherit', 'pipe', 'pipe'],
        /* c8 ignore next */
        shell: runner_js_1.isWindows,
        env: Object.assign(Object.assign({}, node_process_1.env), { FILE: (configs === null || configs === void 0 ? void 0 : configs.parallel) || ((_c = configs === null || configs === void 0 ? void 0 : configs.deno) === null || _c === void 0 ? void 0 : _c.cjs) ? fileRelative : '' }),
    });
    child.stdout.on('data', stdOut);
    child.stderr.on('data', stdOut);
    child.on('close', (code) => __awaiter(void 0, void 0, void 0, function* () {
        end = (0, node_process_1.hrtime)(start);
        const result = code === 0;
        if (showLogs)
            (0, logs_js_1.printOutput)({
                output,
                result,
                configs,
            });
        /* c8 ignore next */
        if (!(yield (0, each_js_1.afterEach)(fileRelative, configs)))
            return false;
        const total = (end[0] * 1e3 + end[1] / 1e6).toFixed(6);
        if (result)
            files_js_1.fileResults.success.set(fileRelative, total);
        else
            files_js_1.fileResults.fail.set(fileRelative, total);
        resolve(result);
    }));
    /* c8 ignore start */
    child.on('error', (err) => {
        end = (0, node_process_1.hrtime)(start);
        const total = (end[0] * 1e3 + end[1] / 1e6).toFixed(6);
        console.error(`Failed to start test: ${filePath}`, err);
        files_js_1.fileResults.fail.set(fileRelative, total);
        resolve(false);
    });
    /* c8 ignore stop */
}));
exports.runTestFile = runTestFile;
