"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.watch = void 0;
const node_fs_1 = require("fs");
const node_path_1 = require("path");
const fs_js_1 = require("../polyfills/fs.js");
const list_files_js_1 = require("../modules/list-files.js");
class Watcher {
    constructor(rootDir, callback) {
        this.files = [];
        this.fileWatchers = new Map();
        this.dirWatchers = new Map();
        this.rootDir = rootDir;
        this.callback = callback;
    }
    watchFile(filePath) {
        /* c8 ignore next */
        if (this.fileWatchers.has(filePath))
            return;
        const watcher = (0, node_fs_1.watch)(filePath, (eventType) => {
            this.callback(filePath, eventType);
        });
        /* c8 ignore start */
        watcher.on('error', () => {
            return;
        });
        /* c8 ignore stop */
        this.fileWatchers.set(filePath, watcher);
    }
    unwatchFiles() {
        for (const [filePath, watcher] of this.fileWatchers) {
            watcher.close();
            this.fileWatchers.delete(filePath);
        }
    }
    watchFiles(filePaths) {
        this.unwatchFiles();
        for (const filePath of filePaths) {
            this.watchFile(filePath);
        }
    }
    watchDirectory(dir) {
        return __awaiter(this, void 0, void 0, function* () {
            /* c8 ignore next */
            if (this.dirWatchers.has(dir))
                return;
            const watcher = (0, node_fs_1.watch)(dir, (_, filename) => __awaiter(this, void 0, void 0, function* () {
                if (filename) {
                    const fullPath = (0, node_path_1.join)(dir, filename);
                    this.files = yield (0, list_files_js_1.listFiles)(this.rootDir);
                    this.watchFiles(this.files);
                    try {
                        const stats = yield (0, fs_js_1.stat)(fullPath);
                        if (stats.isDirectory())
                            yield this.watchDirectory(fullPath);
                        /* c8 ignore start */
                    }
                    catch (_a) { }
                    /* c8 ignore stop */
                }
            }));
            /* c8 ignore start */
            watcher.on('error', () => {
                return;
            });
            /* c8 ignore stop */
            this.dirWatchers.set(dir, watcher);
            const entries = yield (0, fs_js_1.readdir)(dir, { withFileTypes: true });
            for (const entry of entries) {
                if (entry.isDirectory()) {
                    const fullPath = (0, node_path_1.join)(dir, entry.name);
                    yield this.watchDirectory(fullPath);
                }
            }
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const stats = yield (0, fs_js_1.stat)(this.rootDir);
                if (stats.isDirectory()) {
                    this.files = yield (0, list_files_js_1.listFiles)(this.rootDir);
                    this.watchFiles(this.files);
                    yield this.watchDirectory(this.rootDir);
                }
                else
                    this.watchFile(this.rootDir);
                /* c8 ignore start */
            }
            catch (_a) { }
            /* c8 ignore stop */ // c8 bug
        });
    }
    stop() {
        this.unwatchFiles();
        this.unwatchDirectories();
    }
    unwatchDirectories() {
        for (const [dirPath, watcher] of this.dirWatchers) {
            watcher.close();
            this.dirWatchers.delete(dirPath);
        }
    }
}
/* c8 ignore next */ // c8 bug
const watch = (path, callback) => __awaiter(void 0, void 0, void 0, function* () {
    const watcher = new Watcher(path, callback);
    yield watcher.start();
    return watcher;
});
exports.watch = watch;
